@@ -1,7 +1,7 @@
 #ifndef _MOVIT_ALPHA_DIVISION_EFFECT_H
 #define _MOVIT_ALPHA_DIVISION_EFFECT_H 1
 
-// Convert premultiplied alpha to postmultiplied alpha, simply by multiplying.
+// Convert postmultiplied alpha to premultiplied alpha, simply by dividing.
 
 #include <string>
 
@@ -12,8 +12,9 @@ namespace movit {
 class AlphaDivisionEffect : public Effect {
 public:
 	AlphaDivisionEffect() {}
-	virtual std::string effect_type_id() const { return "AlphaDivisionEffect"; }
-	std::string output_fragment_shader();
+	std::string effect_type_id() const override { return "AlphaDivisionEffect"; }
+	std::string output_fragment_shader() override;
+	bool strong_one_to_one_sampling() const override { return true; }
 };
 
 }  // namespace movit
